@@ -1,8 +1,11 @@
-# encoding: utf-8
+"""
+List of all exposed Squirro-APIs.
+"""
 
 import json
 import logging
 import sys
+from typing import Dict, Optional
 
 from ..util import deprecation
 from .communities import CommunitiesMixin
@@ -46,14 +49,13 @@ MAX_UPDATE_COUNT = 10000  # ES hard limit is 10000
 MAX_UPDATE_SIZE = 80 * 1024 * 1024  # 80 MB (nginx has 96 MB limit)
 
 
-class TopicApiBaseMixin(object):
+class TopicApiBaseMixin:
     def get_projects(self):
         """Return all projects."""
         # Build URL
-        url = "%(ep)s/v0/%(tenant)s/projects" % {
-            "ep": self.topic_api_url,
-            "tenant": self.tenant,
-        }
+        url = "{ep}/v0/{tenant}/projects".format(
+            ep=self.topic_api_url, tenant=self.tenant,
+        )
         res = self._perform_request("get", url)
         return self._process_response(res)
 
@@ -71,10 +73,9 @@ class TopicApiBaseMixin(object):
             {'pipelets': [{'id': 'tenant01/textrazor',
                             'name': 'textrazor'}]}
         """
-        url = "%(ep)s/v0/%(tenant)s/pipelets" % {
-            "ep": self.topic_api_url,
-            "tenant": self.tenant,
-        }
+        url = "{ep}/v0/{tenant}/pipelets".format(
+            ep=self.topic_api_url, tenant=self.tenant,
+        )
         res = self._perform_request("get", url)
         return self._process_response(res)
 
@@ -93,11 +94,9 @@ class TopicApiBaseMixin(object):
              'name': 'textrazor',
              'source': 'from squirro.sdk.pipelet import PipeletV1\n\n\n...'}
         """
-        url = "%(ep)s/v0/%(tenant)s/pipelets/%(name)s" % {
-            "ep": self.topic_api_url,
-            "tenant": self.tenant,
-            "name": name,
-        }
+        url = "{ep}/v0/{tenant}/pipelets/{name}".format(
+            ep=self.topic_api_url, tenant=self.tenant, name=name,
+        )
         res = self._perform_request("get", url)
         return self._process_response(res)
 
@@ -111,11 +110,9 @@ class TopicApiBaseMixin(object):
 
             >>> client.delete_pipelet('textrazor')
         """
-        url = "%(ep)s/v0/%(tenant)s/pipelets/%(name)s" % {
-            "ep": self.topic_api_url,
-            "tenant": self.tenant,
-            "name": name,
-        }
+        url = "{ep}/v0/{tenant}/pipelets/{name}".format(
+            ep=self.topic_api_url, tenant=self.tenant, name=name,
+        )
         res = self._perform_request("delete", url)
         self._process_response(res, [204])
 
@@ -134,7 +131,7 @@ class TopicApiBaseMixin(object):
                 "components": [2, 4, 5]
             }
         """
-        url = "%(ep)s/v0/version" % {"ep": self.topic_api_url}
+        url = f"{self.topic_api_url}/v0/version"
         res = self._perform_request("get", url)
         return self._process_response(res, [200])
 
@@ -150,7 +147,7 @@ class TopicApiBaseMixin(object):
         created_before=None,
         created_after=None,
         options=None,
-        **kwargs
+        **kwargs,
     ):
         """Encrypts and signs the `query` and returns it. If set the
         `aggregations`, `created_before`, `created_after`, `fields` and
@@ -198,15 +195,16 @@ class TopicApiBaseMixin(object):
             "created_after": created_after,
         }
         args.update(kwargs)
-        data = dict([(k, v) for k, v in list(args.items()) if v is not None])
+        data = {k: v for k, v in list(args.items()) if v is not None}
 
         res = self._perform_request("post", url, data=json.dumps(data), headers=headers)
         return self._process_response(res)
 
     def query(
         self,
-        project_id,
-        query=None,
+        project_id: str,
+        query: Optional[str] = None,
+        query_context: Optional[Dict[str, Optional[str]]] = None,
         aggregations=None,
         start=None,
         count=None,
@@ -218,15 +216,20 @@ class TopicApiBaseMixin(object):
         options=None,
         encrypted_query=None,
         child_count=None,
-        **kwargs
+        **kwargs,
     ):
         """Returns items for the provided project.
 
         This is the successor to the `get_items` method and should be used in
         its place.
 
+        For information on the item-structure see `Item-Format reference <https://squirro.atlassian.net/wiki/spaces/DOC/pages/4161560/Item+Format>`_
+
         :param project_id: Project identifier.
         :param query: Optional query to run.
+        :param query_context: Dictionary containing context of the query.
+            It includes terms that user typed in a searchbar (`searchbar_query`)
+            and dashboard filters (`dashboard_filters`).
         :param start: Zero based starting point.
         :param count: Maximum number of items to return.
         :param child_count: Maximum number of entities to return with items.
@@ -235,6 +238,8 @@ class TopicApiBaseMixin(object):
             are: `query` (boolean) if True the response will contain highlight
             information. `smartfilters` (list): List of Smart Filter names to
             be highlighted.
+        :param spellcheck: (Dictionary) Apply spellchecking on the given query terms.
+            Spellchecking is performed against the provided field (full elastic field name).
         :param options: Dictionary of options that influence the
             result-set. Valid options are:
 
@@ -259,6 +264,54 @@ class TopicApiBaseMixin(object):
             `created_after`.
         :param kwargs: Additional query parameters. All keyword arguments are
             passed on verbatim to the API.
+
+        **Examples on how to query items**
+
+        *Search for documents and return specific fields*::
+
+            # Possible field values: ["title","body","abstract","keywords","starred","read",
+            #                        "item_id","external_id","created_at","modified_at"]
+            >>> client.query(
+                        project_id="DSuNrcnlSc6x5SJZh02IyQ",
+                        query="virus",
+                        count=1,
+                        fields=["title"])
+            {'count': 1,
+             'items': [{'id': '7rjxIjg_gPjrfjTk3dsTTA',
+               'title': "FDA Adviser: Vaccine To Be OK'd In Days, But 'Normal' May Not Return Until Next Fall",
+               'sources': [{'id': '4fp-1YiASwS-kfNEXYus_g', 'title': 'News Source'}]}],
+             'now': '2022-03-21T16:53:52',
+             'eof': False,
+             'total': 2254,
+             'next_params': {'expected_num_results': 2254, 'start': 1},
+             'query_executed': 'virus',
+             'from_cache': False,
+             'time_ms': 221}
+
+        *Detect spelling errors and provide refined search terms*::
+
+            # Example query `vimus` matches zero items in the index:
+            # --> Suggest similar terms from the index --> return term `virus` to be used for query refinement
+
+            >>> client.query(
+                        project_id="DSuNrcnlSc6x5SJZh02IyQ",
+                        query="vimus",
+                        count=2,
+                        spellcheck={"text":query,"field":"body.en.unstemmed"})
+            {'count': 0,
+             'items': [],
+             'now': '2022-03-21T16:55:38',
+             'eof': True,
+             'total': 0,
+             'next_params': {},
+             'query_executed': 'vimus',
+             'from_cache': False,
+             'spellcheck': [
+                {'original': 'vimus',
+                'corrected': 'virus'}
+                ],
+             'time_ms': 226}
+
         """
         url = "%(ep)s/%(version)s/%(tenant)s" "/projects/%(project_id)s/items/query"
         url = url % {
@@ -272,6 +325,7 @@ class TopicApiBaseMixin(object):
 
         args = {
             "query": query,
+            "query_context": query_context,
             "aggregations": aggregations,
             "start": start,
             "count": count,
@@ -291,7 +345,7 @@ class TopicApiBaseMixin(object):
             )
             kwargs.pop("scroll")
         args.update(kwargs)
-        data = dict([(k, v) for k, v in list(args.items()) if v is not None])
+        data = {k: v for k, v in list(args.items()) if v is not None}
 
         res = self._perform_request("post", url, data=json.dumps(data), headers=headers)
         return self._process_response(res)
@@ -312,7 +366,7 @@ class TopicApiBaseMixin(object):
         query=None,
         aggregations=None,
         method_params=None,
-        **kwargs
+        **kwargs,
     ):
         """Returns recommended items for the provided ids or text.
 
@@ -369,7 +423,7 @@ class TopicApiBaseMixin(object):
             "aggregations": aggregations,
         }
         args.update(kwargs)
-        data = dict((k, v) for k, v in list(args.items()) if v is not None)
+        data = {k: v for k, v in list(args.items()) if v is not None}
 
         res = self._perform_request("post", url, data=json.dumps(data), headers=headers)
         return self._process_response(res)
@@ -782,7 +836,7 @@ class TopicApiBaseMixin(object):
         aggregations=None,
         start=None,
         count=None,
-        **kwargs
+        **kwargs,
     ):
         """Query entity and return aggregations of some entity fields
 
@@ -898,7 +952,7 @@ class TopicApiBaseMixin(object):
             "aggregations": aggregations,
         }
         args.update(kwargs)
-        data = dict([(k, v) for k, v in list(args.items()) if v is not None])
+        data = {k: v for k, v in list(args.items()) if v is not None}
         res = self._perform_request("post", url, data=json.dumps(data), headers=headers)
         return self._process_response(res)
 
@@ -1131,7 +1185,7 @@ class TopicApiBaseMixin(object):
             "created_before": created_before,
             "created_after": created_after,
         }
-        data = dict([(k, v) for k, v in args.items() if v is not None])
+        data = {k: v for k, v in args.items() if v is not None}
 
         items = True
         while items:
@@ -1141,8 +1195,7 @@ class TopicApiBaseMixin(object):
                 )
             )
             items = res.get("items", [])
-            for item in items:
-                yield item
+            yield from items
             if not res.get("eof"):
                 data["next_params"] = res.get("next_params")
             else:
@@ -1403,7 +1456,7 @@ class TopicApiBaseMixin(object):
             item_size = sys.getsizeof(item)
             if item_size > MAX_UPDATE_SIZE:
                 raise ValueError(
-                    "Item size %r > MAX_UPDATE_SIZE %r" % (item_size, MAX_UPDATE_SIZE)
+                    f"Item size {item_size!r} > MAX_UPDATE_SIZE {MAX_UPDATE_SIZE!r}"
                 )
             item_set_size = sys.getsizeof(item_sets[-1])
             if ((item_set_size + item_size) > MAX_UPDATE_SIZE) or (
@@ -1497,10 +1550,13 @@ class TopicApiBaseMixin(object):
             returned. `max_suggestions` should be a non-negative integer.
         :param options: Dictionary of options that influence the result-set.
             Valid options are:
-            - `template_params` dict containing the query template parameters
-            - `select_suggesters` (optional) list containing suggesters to be used.
-                If no suggesters are selected, all suggesters are executed by default.
-                Available suggesters:
+
+                - `template_params` dict containing the query template parameters
+                - `select_suggesters` (optional) list containing suggesters to be used.
+
+            If no suggesters are selected, all suggesters are executed by default.
+            Available suggesters:
+
                 - "facet_value": Complete on facet-values that have "autocomplete" enabled (Exact Search)
                 - "facet_name: Help to find the correct facet
                 - "content_phrase": Complete on key-phrases + item-title (Fuzzy Search) added via NLP-Tagger
@@ -1508,6 +1564,7 @@ class TopicApiBaseMixin(object):
                 - "popular_query": Complete on popular queries on the project, filtered by user_id (optional)
                 - "search_history": Triggered only to pre-populate and show default-suggestions.
                                     Returns last-N searches, filtered by project_id and user_id.
+
         :param filter_query: Squirro query to limit the typeahead suggestions.
             Must be of type `string`. Defaults to `None` if not specified. As
             an example, this parameter can be used to filter the typeahead
@@ -1520,8 +1577,8 @@ class TopicApiBaseMixin(object):
             # Default usage
 
             >>> client.get_typeahead_suggestions(project_id='Sz7LLLbyTzy_SddblwIxaA',
-            ...                                 searchbar_query='Country:India c',
-            ...                                 cursor_pos=15)
+                                             searchbar_query='Country:India c',
+                                             cursor_pos=15)
             {'suggestions': [
                 {'type': 'facetvalue', 'key': 'Country:India
                 City:Calcutta', 'value': 'city:Calcutta', 'score': 12,
@@ -1544,9 +1601,9 @@ class TopicApiBaseMixin(object):
             # - Highlighted tokens are returned as <b> html tags (highlight_cursor_start/end is not used)
 
             >>> client.get_typeahead_suggestions(project_id=project_id,
-            ...         searchbar_query="pla",
-            ...         cursor_pos=0, # cursor_pos not used for this suggester, but parameter is required on API level
-            ...         options={"select_suggesters": ["content_phrase"]})
+                     searchbar_query="pla",
+                     cursor_pos=0, # cursor_pos not used for this suggester, but parameter is required on API level
+                     options={"select_suggesters": ["content_phrase"]})
             {"suggestions":[
                   {
                      "type":"text",
