@@ -5,7 +5,7 @@ log = logging.getLogger(__name__)
 
 
 class ExecutionError(Exception):
-    """Error in executing a machine learning job"""
+    """Error in executing a machine learning job."""
 
     pass
 
@@ -126,7 +126,7 @@ class MLUserFeedbackMixin(object):
         processed=None,
         state=None,
     ):
-        """Modify an existing User Feedback
+        """Modify an existing User Feedback.
 
         :param project_id: Id of the Squirro project.
         :param feedback_id: Id of the User Feedback
@@ -160,7 +160,7 @@ class MLUserFeedbackMixin(object):
         return self._process_response(res, [204])
 
     def delete_feedback(self, project_id, feedback_id):
-        """Delete User Feedback
+        """Delete User Feedback.
 
         :param project_id: Id of the Squirro project.
         :param feedback_id: Id of the User Feedback.
@@ -177,7 +177,7 @@ class MLUserFeedbackMixin(object):
 
         :param project_id: Id of the Squirro project.
         :param model_id: Id of a Model. If it is not given, then the labels of all the
-        the models in the given project are returned.
+                         the models in the given project are returned.
         """
         base_url = "{}/v0/{}/projects/{}/feedbacks/labels"
         headers = {"Content-Type": "application/json"}
